﻿package {
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	import org.undeadcode.console.Color;
	import org.undeadcode.console.Console;
	
	public class Game extends Sprite {
		//----------------------------------------------------------------------------------------------------
		// Constants
		//----------------------------------------------------------------------------------------------------
		
		public static const KeyLeft:uint = 37;
		
		public static const KeyRight:uint = 39;
		
		public static const KeySpace:uint = 32;
		
		public static const KeyEnter:uint = 13;
		
		public static const KeyEscape:uint = 27;
		
		public static const Key1:uint = 49;
		
		public static const Key2:uint = 50;
		
		public static const Key3:uint = 51;
		
		public static const MinTimeInterval:uint = 40;
		
		private static const easyTimeInterval:uint = 200;
		
		private static const normalTimeInterval:uint = 150;
		
		private static const hardTimeInterval:uint = 100;
		
		//----------------------------------------------------------------------------------------------------
		// Fields
		//----------------------------------------------------------------------------------------------------
		
		public static var System:Console;

		public var Points:uint;
		
		private var showMenu:Boolean;
		
		private var menuRendered:Boolean;
		
		private var maxTimeInterval:uint;
		
		private var gamePaused:Boolean;
		
		private var maze:Maze;
		
		private var snake:Snake;
		
		private var hud:Hud;
		
		private var timer:Timer;
		
		private var wait:Boolean;
		
		//----------------------------------------------------------------------------------------------------
		// Public Methods
		//----------------------------------------------------------------------------------------------------
		
		public function Game() {
			if (this.stage) {
				this.initialize();
			} else {
				this.addEventListener(Event.ADDED_TO_STAGE, initialize);
			}
		}
		
		public function GetMaxTimeInterval():uint {
			return this.maxTimeInterval;
		}
		
		public function GetSnake():Snake {
			return this.snake;
		}
		
		public function GetMaze():Maze {
			return this.maze;
		}
		
		public function GetHud():Hud {
			return this.hud;
		}
		
		//----------------------------------------------------------------------------------------------------
		// Private Methods
		//----------------------------------------------------------------------------------------------------
		
		private function initialize(e:Event = null):void {
			this.removeEventListener(Event.ADDED_TO_STAGE, initialize);
			
			this.showMenu = true;
			this.menuRendered = false;
			
			this.maxTimeInterval = 0;
			this.gamePaused = true;
			
			this.maze = new Maze(this);
			this.snake = new Snake(this);
			this.hud = new Hud(this);
			
			this.addEventListener(Event.ENTER_FRAME, frame);
		}
		
		private function frame(e:Event):void {
			if (this.showMenu) {
				this.menuFrame();
			} else {
				this.gameFrame();
			}
		}
		
		private function menuFrame():void {
			if (!this.menuRendered) {
				Game.System.TextColor = Color.BrightYellow;
				Game.System.BackColor = Color.Blue;
				Game.System.Clear();
				
				/* á \xA0   é \x82   í \xA1   ó \xA2   ú \xA3   ñ \xA4   Ñ \xA5   ¿ \xA8   ¡ \xAD */
				Game.System.GotoXY(7, 1);  Game.System.PutString("\xAD\xAD\xADBienvenido al juego del gusano e!!!");
				Game.System.GotoXY(7, 2);  Game.System.PutString("======================================");
				
				Game.System.GotoXY(2, 5);  Game.System.PutString("Esto es muy f\xA0cil. e, el gusano mascota de Pi, tiene que");
				Game.System.GotoXY(2, 6);  Game.System.PutString("comerse a los tomates malignos de Chanquete el Malote.");
				
				Game.System.GotoXY(2, 8);  Game.System.PutString("Para ello no deber , chocar contra la pared electrificada.");
				Game.System.GotoXY(2, 9);  Game.System.PutString("Se puede mover a la derecha o a la izquierda con las flechas,");
				Game.System.GotoXY(2, 10); Game.System.PutString("o dejar que sigua pa'lante, porque es un gusano muy vago, como");
				Game.System.GotoXY(2, 11); Game.System.PutString("yo, y tambi\x82n, porque cuando le apresaron los zombis de");
				Game.System.GotoXY(2, 12); Game.System.PutString("Chanquete, le hicieron escuchar las canciones de Tamara.");
				
				Game.System.GotoXY(2, 14); Game.System.PutString("Que te sea leve... !!!sauj sauj sauJ");
				Game.System.GotoXY(2, 15); Game.System.PutString("Gorka Su\xA0rez Garc\xA1a, 5-12-2000.");
				Game.System.GotoXY(2, 16); Game.System.PutString("(Versi\xA2n flash mejorada del 16-02-2010)");
				Game.System.GotoXY(2, 17); Game.System.PutString("(Versi\xA2n anterior: Beta 1.3 del 11-11-2001)");
				
				Game.System.GotoXY(2, 20); Game.System.PutString("Pulsa para F\xA0cil: 1, Normal: 2, Chungo: 3");
				
				this.menuRendered = true;
			}
			
			if (Game.System.GetKey(Game.Key1)) {
				this.maxTimeInterval = Game.easyTimeInterval;
			} else if (Game.System.GetKey(Game.Key2)) {
				this.maxTimeInterval = Game.normalTimeInterval;
			} else if (Game.System.GetKey(Game.Key3)) {
				this.maxTimeInterval = Game.hardTimeInterval;
			}
			
			if (this.maxTimeInterval > 0) {
				this.newGame();
			}
		}
		
		private function newGame():void {
			Game.System.TextColor = Color.Black;
			Game.System.BackColor = Color.Black;
			Game.System.Clear();
			
			this.Points = 0;
			this.maze.NewGame();
			this.snake.NewGame();
			this.hud.NewGame();
			
			Game.System.TextColor = Color.Magenta;
			Game.System.BackColor = Color.Black;
			
 			Game.System.GotoXY(9, 9);
 			Game.System.PutString("Pulsa una tecla para empezar el martirio...");
 			Game.System.GotoXY(9, 11);
 			Game.System.PutString("Si te arrepientes pulsa ESC para salirte...");
			
			this.showMenu = false;
			this.gamePaused = true;
			this.setWait();
		}
		
		private function setWait():void {
			this.wait = true;
			this.timer = new Timer(200, 0);
			this.timer.addEventListener(TimerEvent.TIMER, waitNewGame);
			this.timer.start();
		}
		
		private function waitNewGame(event:TimerEvent):void {
			this.timer.removeEventListener(TimerEvent.TIMER, waitNewGame);
			this.timer.stop();
			this.wait = false;
		}
		
		private function gameFrame():void {
			if (this.gamePaused) {
				if (!this.wait && Game.System.KeyboardPressed) {
					if (!this.snake.Alive) {
						this.showMenu = true;
						this.menuRendered = false;
						this.maxTimeInterval = 0;
					} else {
						this.gamePaused = false;
					}
				}
			} else {
				this.maze.Frame();
				this.hud.Frame();
				this.snake.Frame();
				
				if (!this.snake.Alive) {
					this.showFinalScore();
					this.gamePaused = true;
					this.setWait();
				}
			}
		}
		
		private function showFinalScore():void {
			Game.System.TextColor = Color.BrightYellow;
			Game.System.BackColor = Color.Magenta;
			
			/* á \xA0   é \x82   í \xA1   ó \xA2   ú \xA3   ñ \xA4   Ñ \xA5   ¿ \xA8   ¡ \xAD */
			Game.System.GotoXY(2, 3); Game.System.PutString("                                                                  ");
			Game.System.GotoXY(2, 4); Game.System.PutString("  Pobre e, pero todo el mundo ha de morir algun d\xA1a, \xA8no?         ");
			Game.System.GotoXY(2, 5); Game.System.PutString("  La pr\xA2xima vez se m\xA0s h\xA0bil, \xAD\xAD\xADpecadorrr!!!                    ");
			
			Game.System.GotoXY(2, 6);
			if (this.Points < 1) {
				Game.System.PutString("  Tienes " + this.Points.toString() + " puntos, yo que tu me cambiaba de identidad por honor.  ");
			} else if (this.Points < 10) {
				Game.System.PutString("  Tu catastr\xA2fica puntuaci\xA2n ha sido: " + this.Points.toString() + "                           ");
			} else if (this.Points < 20) {
				Game.System.PutString("  Tu cutre puntuaci\xA2n ha sido: " + this.Points.toString() + "                                 ");
			} else if (this.Points < 30) {
				Game.System.PutString("  Tu mediocre puntuaci\xA2n ha sido: " + this.Points.toString() + "                              ");
			} else if (this.Points < 40) {
				Game.System.PutString("  Tu puntuaci\xA2n ha sido: " + this.Points.toString() + "                                       ");
			} else if (this.Points < 60) {
				Game.System.PutString("  Tu puntuaci\xA2n de viciado ha sido: " + this.Points.toString() + "                            ");
			} else if (this.Points < 70) {
				Game.System.PutString("  Tu puntuaci\xA2n de super-viciado ha sido: " + this.Points.toString() + "                      ");
			} else if (this.Points < 80) {
				Game.System.PutString("  Tu puntuaci\xA2n de mega-viciator ha sido: " + this.Points.toString() + "                      ");
			} else if (this.Points < 90) {
				Game.System.PutString("  Tu puntuaci\xA2n de ultra-viciator-plus-plus ha sido: " + this.Points.toString() + "           ");
			} else if (this.Points < 100) {
				Game.System.PutString("  Tu puntuaci\xA2n de super-mega-ultra-viciator++ ha sido: " + this.Points.toString() + "        ");
			} else {
				Game.System.PutString("  Tu puntuaci\xA2n de Master-Viciator ha sido: " + this.Points.toString() + "                   ");
			}
			
			Game.System.GotoXY(2, 7); Game.System.PutString("                                                                  ");
			
			Game.System.TextColor = Color.White;
			Game.System.BackColor = Color.Black;
			Game.System.GotoXY(0, 22); Game.System.PutString("Pulsa una tecla para continuar...");
		}
	}
}