﻿package {
	import flash.geom.Point;
	import org.undeadcode.console.Color;
	import org.undeadcode.console.Console;
	
	public class Snake {
		//----------------------------------------------------------------------------------------------------
		// Constants
		//----------------------------------------------------------------------------------------------------
		
		public static const North:uint = 1;
		public static const East:uint  = 2;
		public static const South:uint = 3;
		public static const West:uint  = 4;
		
		//----------------------------------------------------------------------------------------------------
		// Fields
		//----------------------------------------------------------------------------------------------------
		
		private var game:Game;
		
		private var direction:uint;
		
		private var body:Vector.<Point>;
		
		private var time:uint;
		
		private var leftKeyPressed:Boolean;
		
		private var rightKeyPressed:Boolean;
		
		public var Alive:Boolean;
		
		//----------------------------------------------------------------------------------------------------
		// Public Methods
		//----------------------------------------------------------------------------------------------------
		
		public function Snake(g:Game) {
			this.game = g;
		}
		
		public function NewGame():void {
			this.Alive = true;
			this.time = 0;
			this.leftKeyPressed = false;
			this.rightKeyPressed = false;
			this.direction = Snake.West;
			this.body = new Vector.<Point>();
			
			this.body.push(new Point(30, 10));
			for (var i:uint = 0; i < 3; i++) {
				this.body.push(new Point(this.body[i].x + 1, this.body[i].y));
			}
			
			this.Draw();
		}
		
		public function Draw():void {
			Game.System.TextColor = Color.Green;
			Game.System.BackColor = Color.Black;
			
			for each (var part:Point in this.body) {
				Game.System.GotoXY(part.x, part.y);
				Game.System.PutChar("*");
			}
		}
		
		public function Frame():void {
			this.Draw();
			this.checkInput();
				
			this.time += Game.MinTimeInterval;
			if (this.time >= this.game.GetMaxTimeInterval()) {
				this.time -= this.game.GetMaxTimeInterval();
				this.moveSnake();
			}
		}
		
		public function Collides(p:Point):Boolean {
			for each (var part:Point in this.body) {
				if (part.x == p.x && part.y == p.y) return true;
			}
			return false;
		}
		
		//----------------------------------------------------------------------------------------------------
		// Private Methods
		//----------------------------------------------------------------------------------------------------
		
		private function moveSnake():void {
			switch(this.direction) {
				case Snake.North:
					this.move(0, -1);
					break;
				case Snake.East:
					this.move(1, 0);
					break;
				case Snake.South:
					this.move(0, 1);
					break;
				case Snake.West:
					this.move(-1, 0);
					break;
			}
		}
		
		private function goForward(p:Point):void {
			for (var i:int = this.body.length - 1; i > 0 ; i--) {
				this.body[i] = this.body[i - 1];
			}
			this.body[0] = p;
		}
		
		private function move(incx:int, incy:int):void {
			var p:Point = new Point();
			p.x = this.body[0].x + incx;
			p.y = this.body[0].y + incy;
			
			if (this.Collides(p) || this.game.GetMaze().CollidesWalls(p)) {
				this.Alive = false;
				
			} else if (this.game.GetMaze().CollidesTomatoes(p)) {
				this.game.GetMaze().KillTomato(p);
				this.goForward(p);
				p = this.body[this.body.length - 1];
				this.body.push(new Point(p.x, p.y));
				
			} else {
				this.goForward(p);
			}
		}
		
		private function checkInput():void {
			if (Game.System.GetKey(Game.KeyLeft)) {
				if (!this.leftKeyPressed) {
					this.leftKeyPressed = true;
					
					switch(this.direction) {
						case Snake.North:
							this.direction = Snake.West;
							break;
						case Snake.East:
							this.direction = Snake.North;
							break;
						case Snake.South:
							this.direction = Snake.East;
							break;
						case Snake.West:
							this.direction = Snake.South;
							break;
					}
				}
				
			} else if (Game.System.GetKey(Game.KeyRight)) {
				if (!this.rightKeyPressed) {
					this.rightKeyPressed = true;
					
					switch(this.direction) {
						case Snake.North:
							this.direction = Snake.East;
							break;
						case Snake.East:
							this.direction = Snake.South;
							break;
						case Snake.South:
							this.direction = Snake.West;
							break;
						case Snake.West:
							this.direction = Snake.North;
							break;
					}
				}
				
			} else if (Game.System.GetKey(Game.KeyEscape)) {
				this.game.GetHud().DrawSuicideMsg();
				
			} else if (Game.System.KeyboardPressed) {
				this.game.GetHud().DrawWrongKeyMsg();
			} else {
				if (this.leftKeyPressed && !Game.System.GetKey(Game.KeyLeft)) {
					this.leftKeyPressed = false;
				}
				
				if (this.rightKeyPressed && !Game.System.GetKey(Game.KeyRight)) {
					this.rightKeyPressed = false;
				}
			}
		}
	}
}