﻿package org.undeadcode.console {
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.geom.ColorTransform;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	public class Console extends Sprite {
		//----------------------------------------------------------------------------------------------------
		// Constants
		//----------------------------------------------------------------------------------------------------
		
		public static const MaxCols:uint = 80;
		
		public static const MaxRows:uint = 25;
		
		private static const maxKeyboardIndex:uint = 256;
		
		private static const nextLineChar:uint = 10; // '\n'
		
		private static const returnChar:uint = 13; // '\r'
		
		private static const backspaceChar:uint = 8; // '\b'
		
		private static const fontWidth:uint = 8;
		
		private static const fontHeight:uint = 12;
		
		//----------------------------------------------------------------------------------------------------
		// Fields
		//----------------------------------------------------------------------------------------------------
		
		[Embed(source = 'Font.png')]
		private static var fontFile:Class;
		
		private var textBuffer:Bitmap;
		
		private var backBuffer:Bitmap;
		
		private var texture:Bitmap;
		
		private var keyboard:Vector.<Boolean>;
		
		private var lastKey:uint;
		
		private var keyboardPressed:Boolean;
		
		private var input:Vector.<uint>;
		
		private var output:Vector.<Vector.<uint>>;
		
		private var outputTextColor:Vector.<Vector.<uint>>;
		
		private var outputBackColor:Vector.<Vector.<uint>>;
		
		private var cursorX:uint;
		
		private var cursorY:uint;
		
		private var auxPoint:Point;
		
		private var auxRectangle:Rectangle;
		
		private var auxColorRectangle:Rectangle;
		
		private var textColor:uint;
		
		private var backColor:uint;
		
		//----------------------------------------------------------------------------------------------------
		// Public Methods
		//----------------------------------------------------------------------------------------------------
		
		public function Console() {
			if (this.stage) {
				this.initialize();
			} else {
				this.addEventListener(Event.ADDED_TO_STAGE, initialize);
			}
		}
		
		public function GotoXY(cx:uint, cy:uint):void {
			if (cx < Console.MaxCols) this.cursorX = cx;
			if (cy < Console.MaxRows) this.cursorY = cy;
		}
		
		public function get KeyboardPressed():Boolean {
			return this.keyboardPressed;
		}
		
		public function get TextColor():uint {
			return this.textColor;
		}
		
		public function get BackColor():uint {
			return this.backColor;
		}
		
		public function set TextColor(value:uint):void {
			if(value < Color.MaxColors) {
				this.textColor = value;
			}
		}
		
		public function set BackColor(value:uint):void {
			if(value < Color.MaxColors) {
				this.backColor = value;
			}
		}
		
		//----------------------------------------------------------------------------------------------------
		// Public Methods (Clear)
		//----------------------------------------------------------------------------------------------------
		
		public function Clear():void {
			this.cursorX = 0;
			this.cursorY = 0;
			
			for (var i:int = 0; i < Console.MaxRows; i++) {
				for (var j:int = 0; j < Console.MaxCols; j++) {
					this.output[i][j] = 0;
					this.outputTextColor[i][j] = this.TextColor;
					this.outputBackColor[i][j] = this.BackColor;
				}
			}
		}
		
		public function Flush():void {
			this.input.splice(0, this.input.length);
		}
		
		public function FlushAll():void {
			this.Flush();
			this.ResetKeyboard();
		}
		
		public function ResetKeyboard():void {
			for (var i:int = 0; i < Console.maxKeyboardIndex; i++) {
				this.keyboard[i] = false;
			}
			this.lastKey = 0;
			this.keyboardPressed = false;
		}
		
		//----------------------------------------------------------------------------------------------------
		// Public Methods (Output)
		//----------------------------------------------------------------------------------------------------
		
		public function PutString(s:String):void {
			for (var i:int = 0; i < s.length; i++) {
				this.PutCharCode(s.charCodeAt(i));
			}
		}
		
		public function PutChar(c:String):void {
			this.PutCharCode(c.charCodeAt(0));
		}
		
		public function PutCharCode(code:uint):void {
			if (code == Console.nextLineChar) {
				this.gotoNextLine();
				
			} else if (code == Console.backspaceChar) {
				if (this.cursorX > 0) {
					this.cursorX--;
				}
				
			} else {
				this.output[this.cursorY][this.cursorX] = code;
				this.outputTextColor[this.cursorY][this.cursorX] = this.TextColor;
				this.outputBackColor[this.cursorY][this.cursorX] = this.BackColor;
				this.cursorX++;
				if (this.cursorX >= Console.MaxCols) {
					this.gotoNextLine();
				}
			}
		}
		
		//----------------------------------------------------------------------------------------------------
		// Public Methods (Input)
		//----------------------------------------------------------------------------------------------------
		
		public function GetKey(index:uint):Boolean {
			if (index < Console.maxKeyboardIndex) {
				return this.keyboard[index];
			} else {
				return false;
			}
		}
		
		public function GetChar():String {
			var key:uint = this.input.shift();
			return String.fromCharCode(key);
		}
		
		public function GetLine():String {
			var line:String = "";
			var ended:Boolean = false;
			var key:uint;
			
			while (this.input.length > 0 && !ended) {
				key = this.input.shift();
				if (key == Console.returnChar) {
					ended = true;
				} else if (key == Console.backspaceChar) {
					if (line.length > 0) {
						line = line.substr(0, line.length - 1);
					}
				} else {
					line = line + String.fromCharCode(key);
				}
			}
			return line;
		}
		
		//----------------------------------------------------------------------------------------------------
		// Private Methods
		//----------------------------------------------------------------------------------------------------
		
		private function initialize(e:Event = null):void {
			this.removeEventListener(Event.ADDED_TO_STAGE, initialize);
			
			this.auxPoint = new Point();
			this.auxRectangle = new Rectangle(0, 0, Console.fontWidth, Console.fontHeight);
			this.auxColorRectangle = new Rectangle(0, 0, Console.fontWidth, Console.fontHeight);
			
			this.TextColor = Color.White;
			this.BackColor = Color.Black;
			
			this.texture = new Console.fontFile() as Bitmap;
			this.textBuffer = new Bitmap(new BitmapData(640, 300, true, 0));
			this.backBuffer = new Bitmap(new BitmapData(640, 300, false, 0));
			
			this.keyboard = new Vector.<Boolean>();
			this.input = new Vector.<uint>();
			this.output = new Vector.<Vector.<uint>>();
			this.outputTextColor = new Vector.<Vector.<uint>>();
			this.outputBackColor = new Vector.<Vector.<uint>>();
			
			for (var i:int = 0; i < Console.MaxRows; i++) {
				this.output[i] = new Vector.<uint>();
				this.outputTextColor[i] = new Vector.<uint>();
				this.outputBackColor[i] = new Vector.<uint>();
				for (var j:int = 0; j < Console.MaxCols; j++) {
					this.output[i][j] = 0;
					this.outputTextColor[i][j] = this.TextColor;
					this.outputBackColor[i][j] = this.BackColor;
				}
			}
			
			this.ResetKeyboard();
			
			this.addEventListener(Event.ENTER_FRAME, render);
			this.stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown);
			this.stage.addEventListener(KeyboardEvent.KEY_UP, keyUp);
			
			this.addChild(this.backBuffer);
			this.addChild(this.textBuffer);
		}
		
		private function render(e:Event):void {
			var c:uint;
			
			for (var i:int = 0; i < Console.MaxRows; i++) {
				for (var j:int = 0; j < Console.MaxCols; j++) {
					this.auxPoint.x = Console.fontWidth * j;
					this.auxPoint.y = Console.fontHeight * i;
			
					c = this.output[i][j];
					this.auxRectangle.x = Console.fontWidth * (c % 16);
					this.auxRectangle.y = Console.fontHeight * Math.floor(c / 16);
					
					this.auxColorRectangle.x = auxPoint.x;
					this.auxColorRectangle.y = auxPoint.y;
					this.backBuffer.bitmapData.fillRect(this.auxColorRectangle, Color.ArgbValues[this.outputBackColor[i][j]]);
					this.textBuffer.bitmapData.copyPixels(this.texture.bitmapData, this.auxRectangle, this.auxPoint);
					this.textBuffer.bitmapData.colorTransform(this.auxColorRectangle, Color.Transforms[this.outputTextColor[i][j]]);
				}
			}
		}
		
		private function gotoNextLine():void {
			this.cursorX = 0;
			this.cursorY++;
			if (this.cursorY >= Console.MaxRows) {
				this.moveAllUp();
				this.cursorY--;
			}
		}
		
		private function moveAllUp():void {
			var i:int, j:int;
			
			for (i = 0; i < Console.MaxRows - 1; i++) {
				for (j = 0; j < Console.MaxCols; j++) {
					this.output[i][j] = this.output[i + 1][j];
				}
			}
			
			for (j = 0; j < Console.MaxCols; j++) {
				this.output[i][j] = 0;
			}
		}
		
		private function setKeyboard(index:uint, value:Boolean):void {
			if (index < Console.maxKeyboardIndex) {
				this.keyboard[index] = value;
			}
		}
		
		private function keyDown(e:KeyboardEvent):void {
			this.lastKey = e.keyCode;
			this.setKeyboard(e.keyCode, true);
			this.keyboardPressed = true;
			if (e.charCode > 0) {
				this.input.push(e.charCode);
			}
		}
		
		private function keyUp(e:KeyboardEvent):void {
			this.setKeyboard(e.keyCode, false);
			this.keyboardPressed = false;
		}
	}
}